#
# vocalips020.tcl - GUI
# 
# Rev.0.20  Nov. 23, 2007
# Rev.0.10  Nov. 3, 2007
# 
# author: zhuo   www.geocities.jp/zhuoware
# 

set VocalipsRevision "0.20"

set w .top
catch {destroy $w}
toplevel $w
wm title $w "VOCALIPS Rev.$VocalipsRevision"
wm iconname $w "VOCALIPS Rev.$VocalipsRevision"
wm withdraw .

proc createFileEntry { path text cmd } {
    frame $path
    label $path.lab -width 16 -text $text -anchor e
    entry $path.ent -width 80
    button $path.but -text "Browse ..." -command "$cmd $path"
    pack $path.lab -side left
    pack $path.ent -side left -expand yes -fill x
    pack $path.but -side left
    pack $path -fill x -padx 1c -pady 3
}

proc setEntryString { path string } {
    if {[string compare $string ""]} {
	$path delete 0 end
	$path insert 0 $string
	$path xview end
    }
}

if { 0 } {
    frame $w.cons
    scrollbar $w.cons.scroll -command "$w.cons.text yview"
    text $w.cons.text -height 10 -yscroll "$w.cons.scroll set"
    pack $w.cons.scroll -side right -expand no -fill y
    pack $w.cons.text -side left -expand yes -fill both
    pack $w.cons -side bottom  -expand yes -fill both
}

proc consPuts { str } {
    global w
    catch {$w.cons.text insert end $str}
}


createFileEntry $w.vsqfile "vocaloid file: " "vsqFileChooser $w.outfile.ent"
createFileEntry $w.outfile "output file: " "outFileChooser"
createFileEntry $w.tblfile "phoneme conv. file: " "tblFileChooser"

$w.tblfile.ent delete 0 end
$w.tblfile.ent insert 0 [lindex [glob -nocomplain -types f "*.tbl"] 0] 
$w.tblfile.ent xview end

frame $w.settings

labelframe $w.settings.tracks -text "track chooser" -padx 5 -pady 5
scrollbar $w.settings.tracks.scroll -command "$w.settings.tracks.list yview"
listbox $w.settings.tracks.list -yscroll "$w.settings.tracks.scroll set" -setgrid 1 -height 8 -selectmode extended
pack $w.settings.tracks.scroll -side right -fill y
pack $w.settings.tracks.list -side left -expand yes -fill both 
pack $w.settings.tracks -side left -expand yes -fill both  -padx 3m


labelframe $w.settings.mode -text "output mode" -padx 5 -pady 5
radiobutton $w.settings.mode.e2 -text "standard" -variable outputMode -relief flat -value "e2" -anchor w
radiobutton $w.settings.mode.e1 -text "event-base, \"sil\" when early" -variable outputMode -relief flat -value "e1" -anchor w
radiobutton $w.settings.mode.e0 -text "event-base, always with \"end\"" -variable outputMode -relief flat -value "e0" -anchor w
radiobutton $w.settings.mode.f -text "frame-base" -variable outputMode -relief flat -value "f" -anchor w
radiobutton $w.settings.mode.v -text "raw vsq text" -variable outputMode -relief flat -value "v" -anchor w
pack $w.settings.mode.e2 $w.settings.mode.e1 $w.settings.mode.e0 $w.settings.mode.f $w.settings.mode.v -side top -pady 2 -expand no -fill x
pack $w.settings.mode -side left -expand no -fill y  -padx 3m
$w.settings.mode.e2 invoke

labelframe $w.settings.fb -text "options for frame-base mode" -padx 5 -pady 5
label $w.settings.fb.fpslab -text "fps"
entry $w.settings.fb.fpsent -width 8
$w.settings.fb.fpsent insert 0 "30.00"
label $w.settings.fb.offsetlab -text "offset(msec)"
entry $w.settings.fb.offsetent -width 8
$w.settings.fb.offsetent insert 0 "0.0"
pack $w.settings.fb.fpslab $w.settings.fb.fpsent $w.settings.fb.offsetlab $w.settings.fb.offsetent -side left -expand yes -padx 2m
pack $w.settings.fb -side bottom -fill none -padx 3m

pack $w.settings -side top -expand yes -fill both


frame $w.buttons
button $w.buttons.convert -width 16 -text "convert" -command "convert $w"
button $w.buttons.quit -width 16 -text quit -command "destroy $w; exit"
pack $w.buttons.convert -side left -expand 1
pack $w.buttons.quit -side left -expand 1
pack $w.buttons -side bottom -fill x -pady 5m


proc getSelectedTracks {} {
    global w
    return [$w.settings.tracks.list curselection]
}


proc setTrackList {vsqfile} {
    global w
    set listpath $w.settings.tracks.list
    if { $vsqfile == "" || ! [file exists $vsqfile] } {
	tk_messageBox -icon info -message "Please choose a vocaloid file." -type ok -parent $w
	return
    }
    file copy -force "$vsqfile" "T.VSQ"
    exec bin/mf2t.exe "T.VSQ" | bin/Perl.exe bin/vsqtxt2lipsync_041.pl -m t > "T1.TXT"
    file delete "T.VSQ"
    set fd [open "T1.TXT"]
    $listpath delete 0 end
    while {[gets $fd line] >= 0 } {
	$listpath insert end $line	
    }
    close $fd
    file delete "T1.TXT"
    $listpath selection set 0 end
}


proc vsqFileChooser { out_ent path } {
    set types {{"vocaloid files" {.vsq}} {"All files" *}}
    set file [tk_getOpenFile -filetypes $types -parent $path -initialfile [$path.ent get]]
    regsub {\.vsq$} $file ".txt" outfile 
    setEntryString $path.ent $file
    setEntryString $out_ent $outfile

    setTrackList $file
}

proc tblFileChooser { path } {
    set types {{"table files" {.tbl}} {"All files" *}}
    setEntryString $path.ent [tk_getOpenFile -filetypes $types -parent $path -initialfile [$path.ent get]]
}

proc outFileChooser { path } {
    set types {{"text files" {.txt}} {"All files" *}}
    setEntryString $path.ent [tk_getSaveFile -filetypes $types -parent $path -initialfile [$path.ent get]]
}



proc convert { w } {
    global outputMode
    set vsqfile [$w.vsqfile.ent get]
    set tblfile [$w.tblfile.ent get]
    set outfile [$w.outfile.ent get]
    set fps [$w.settings.fb.fpsent get]
    set offset [$w.settings.fb.offsetent get]
    set tracks [join [getSelectedTracks] ","]
    if { $vsqfile == "" || ! [file exists $vsqfile] } {
	tk_messageBox -icon info -message "Please choose a vocaloid file." -type ok -parent $w
	return
    }
    if { $tblfile == "" || ! [file exists $tblfile] } {
	tk_messageBox -icon info -message "Please choose a table file." -type ok -parent $w
	return
    }
    if { $outfile == "" } {
	tk_messageBox -icon info -message "Please choose an output file." -type ok -parent $w
	return
    }
    if { $outputMode == "" } {
	tk_messageBox -icon info -message "Please choose an output mode." -type ok -parent $w
	return
    }
    file copy -force "$vsqfile" "T.VSQ"
##    exec bin/mf2t.exe "T.VSQ" "T1.TXT"
##    exec bin/Perl.exe bin/vsqtxt2lipsync_041.pl -m "$outputMode" -p "$tblfile" "$fps" "$offset" < "T1.TXT" > "T2.TXT"
    exec bin/mf2t.exe "T.VSQ" | bin/Perl.exe bin/vsqtxt2lipsync_041.pl -t "$tracks" -m "$outputMode" -p "$tblfile" "$fps" "$offset" > "T2.TXT"

    if { (![file exists "$outfile"]) || ([tk_messageBox -icon question -message "file exists: overwrite?  $outfile" -type okcancel -parent $w] == "ok") } {
	file copy -force "T2.TXT" "$outfile"
    }
    file delete "T.VSQ" "T2.TXT"
    tk_messageBox -icon info -message "$outfile created.\n" -type ok -parent $w
}
